package org.uchile.bdd;

import java.io.IOException;
import java.io.PrintWriter;
import java.sql.Connection;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.Statement;

import javax.naming.InitialContext;
import javax.servlet.ServletConfig;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.sql.DataSource;


/**
 * Simple Hello servlet.
 */

public final class BdDServlet extends HttpServlet {

	private static DataSource datasource = null;

	/**
	 * Read initial config data from web.xml for this servlet
	 */
	@SuppressWarnings("unused")
	public void init(ServletConfig config) throws ServletException {

		try
		{
			InitialContext initialContext = new InitialContext();
			if ( initialContext == null )
			{
				String message = "There was no InitialContext in DBBroker. We're about to have some problems.";
				System.err.println("*** " + message);
				throw new Exception(message);
			}

			// actual jndi name is "jdbc/postgres"
			datasource = (DataSource) initialContext.lookup( "java:/comp/env/jdbc/postgres" );

			if ( datasource == null )
			{
				String message = "Could not find our DataSource in DBBroker. We're about to have problems.";
				System.err.println("*** " + message);
				throw new Exception(message);
			}
		}
		catch (Exception e)
		{
			throw new ServletException(e.getMessage());
		}
	}

	public static synchronized Connection getConnection() 
			throws SQLException
	{
		return datasource.getConnection();
	}

	public static synchronized void freeConnection(Connection connection)
	{
		try
		{
			connection.close();
		}
		catch (Exception e)
		{
			System.err.println("DBBroker: Threw an exception closing a database connection");
			e.printStackTrace();
		}
	}

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;

	/**
	 * Forward POST data to GET in this case
	 */
	public void doPost(HttpServletRequest request,
			HttpServletResponse response)
					throws IOException, ServletException {
		doGet(request,response);
	}

	/**
	 * Respond to a GET request for the content produced by
	 * this servlet.
	 *
	 * @param request The servlet request we are processing
	 * @param response The servlet response we are producing
	 *
	 * @exception IOException if an input/output error occurs
	 * @exception ServletException if a servlet error occurs
	 */
	public void doGet(HttpServletRequest request,
			HttpServletResponse response)
					throws IOException, ServletException {

		String consulta = request.getParameter("consulta");

		response.setContentType("text/html");
		PrintWriter writer = response.getWriter();        
		writer.println("<html>");
		writer.println("<head>");
		writer.println("<title>Base de Datos</title>");
		writer.println("</head>");
		writer.println("<body bgcolor=white>");

		writer.println("<table border=\"0\" cellpadding=\"10\">");
		writer.println("<tr>");
		writer.println("<td>");
		writer.println("<img src=\"images/good.png\">");
		writer.println("</td>");
		writer.println("<td>");
		writer.println("<h1>Ejemplo dinamico con bdd.</h1>");

		if(consulta!=null){
			writer.println("<h1>Consulta: "+Utils.escapeHTML(consulta)+"</h1>");
			writer.println("<h1>Resultados:</h1>");
			Statement stmt = null;
			Connection conn = null;
			try {
				conn = getConnection();
				stmt = conn.createStatement();
				ResultSet rs = stmt.executeQuery(consulta);
				writer.println(Utils.resultSetToHtml(rs));
			} catch (SQLException e) {
				writer.println("<h1>Error:"+Utils.escapeHTML(e.getMessage())+"</h1>");
				writer.println("<p>");
				e.printStackTrace(writer);
				writer.println("</p>");
			} finally {
				try{ stmt.close(); }catch(Exception e){};
				try{ conn.close(); }catch(Exception e){};
			}
		}
		writer.println("</td>");
		writer.println("</tr>");
		writer.println("</table>");

		writer.println("<form action=\"bdd\" method=\"POST\">");
		writer.println("Ejemplo: intenta con 'SELECT * FROM test.hello;'<br />");
		writer.println("Consulta: <input type=\"text\" name=\"consulta\"> ");
		writer.println("<input type=\"submit\" value=\"Submit\" />");
		writer.println("</form>");

		writer.println("BdDServlet.java");

		writer.println("</body>");
		writer.println("</html>");
	}
} 